#define WNCK_I_KNOW_THIS_IS_UNSTABLE 1
#include <libwnck/screen.h>
#include <libwnck/window.h>
#include <gtk/gtk.h>
#include <stdio.h>
#include "injectso.h"

enum {
	APPLICATION_NAME,
	APPLICATION_PID,
	NB_COLUMNS
};

void on_destroy(GtkWidget* widget, gpointer data)
{
    gtk_main_quit();
}

/* The file selection widget and the string to store the chosen filename */

void store_filename(GtkButton *button, gpointer user_data)
{
   const gchar *selected_filename;
   GtkFileSelection* file_selector = GTK_FILE_SELECTION(user_data);
   int pid = GPOINTER_TO_INT(g_object_get_data(G_OBJECT(file_selector), "pid"));
   
   selected_filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(file_selector));
   injectso(pid, "libtest.so", selected_filename);
}

void dump_glade_file(int pid)
{
   GtkWidget *file_selector;

   /* Create the selector */
   file_selector = gtk_file_selection_new("Please select a file to store the GUI dump.");
   
   g_object_set_data(G_OBJECT(file_selector), "pid", GINT_TO_POINTER(pid));
   g_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),
					"clicked",
					G_CALLBACK(store_filename),
					(gpointer) file_selector);
   			   
   /* Ensure that the dialog box is destroyed when the user clicks a button. */
   g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->ok_button),
							"clicked",
							G_CALLBACK(gtk_widget_destroy), 
                            (gpointer) file_selector); 

   g_signal_connect_swapped(GTK_OBJECT(GTK_FILE_SELECTION(file_selector)->cancel_button),
							"clicked",
							G_CALLBACK(gtk_widget_destroy),
                            (gpointer) file_selector); 

   /* Display that dialog */
   gtk_widget_show(file_selector);
}

void on_row_activated(GtkTreeView* tree_view, GtkTreePath* path, GtkTreeViewColumn *column)
{
	GtkTreeIter iter;
	GtkTreeModel *model;
	int pid;
	
	model = gtk_tree_view_get_model(tree_view);
  
	gtk_tree_model_get_iter(model, &iter, path);
	gtk_tree_model_get(GTK_TREE_MODEL(model), &iter, APPLICATION_PID, &pid, -1);

	if (pid > 0)
		dump_glade_file(pid);
}

int main(int argc, char* argv[])
{
	GList* windows;
	GList* l;
	WnckScreen* screen;
	GtkListStore* list_store;
	GtkCellRenderer *renderer;
	GtkTreeViewColumn *column;
	GtkTreeView* tree;
	GtkVBox* vbox;
	GtkLabel* label;
	GtkWindow* main_window;

	gtk_init(&argc, &argv);

	list_store = gtk_list_store_new(NB_COLUMNS, G_TYPE_STRING, G_TYPE_INT);
	tree = GTK_TREE_VIEW(gtk_tree_view_new_with_model(GTK_TREE_MODEL(list_store)));
	renderer = gtk_cell_renderer_text_new();
	column = gtk_tree_view_column_new_with_attributes("Applications", renderer, "text", 0, NULL);
	gtk_tree_view_append_column(GTK_TREE_VIEW(tree), column);

	label = GTK_LABEL(gtk_label_new("Double click in an application\nto dump its graphical interface."));
	vbox = GTK_VBOX(gtk_vbox_new(FALSE, 4));
	main_window = GTK_WINDOW(gtk_window_new(GTK_WINDOW_TOPLEVEL));
	
	screen = wnck_screen_get_default();
	wnck_screen_force_update(screen);		
	windows = wnck_screen_get_windows(screen);
  
	l = windows;
	while (l != NULL)
    {
		WnckWindow* win = WNCK_WINDOW(l->data);
		int pid = wnck_window_get_pid(win);

		if (pid)
		{
			GtkTreeIter   iter;
			const char* name = wnck_window_get_name(win);
			
			gtk_list_store_append(list_store, &iter);
			gtk_list_store_set(list_store, &iter, APPLICATION_NAME, name, APPLICATION_PID, pid, -1);
		}

		l = l->next;
    }

	g_signal_connect(G_OBJECT(main_window), "destroy", G_CALLBACK (on_destroy), NULL);
	g_signal_connect(G_OBJECT(tree), "row_activated", G_CALLBACK (on_row_activated), NULL);

	gtk_container_set_border_width(GTK_CONTAINER(vbox), 4);
	gtk_box_pack_start_defaults(GTK_BOX(vbox), GTK_WIDGET(label));
	gtk_box_pack_start_defaults(GTK_BOX(vbox), GTK_WIDGET(tree));
	
	gtk_container_add(GTK_CONTAINER(main_window), GTK_WIDGET(vbox));
	gtk_widget_show_all(GTK_WIDGET(main_window));

	gtk_main();

	return 0;
}
